;------ rf12_trans -------------------------------------------------------------
;
;	Sender: Daten werden mit XL bertragen und im RF gespeichert,
;	Empfnger: Gespeicherte Daten werden in XL geschoben,
;	XH ist dabei jeweils 0xB0.  
;
;-------------------------------------------------------------------------------
rf12_trans:
	cbi		RF_PORT, CS			;CS auf 0(aktiv)
	ldi		temp, 16			;16 Datenbits 

rf12_trans_loop:
	lsl		XL					;Ins derzeitige LSB 0 schreiben und schieben
	rol		XH					;Weiterschieben und MSB ins Carry
	cbi		RF_PORT, SDI		;SDI auf 0
	brcc	rf12_trans_0		;Wenn Carry schon leer ist, so lassen
	sbi		RF_PORT, SDI		;Andernfalls SDI auf 1

rf12_trans_0:
	sbic	RF_PIN, SDO			;Wenn SDO auf 1
	ori		XL, 1				;Ins derzeitige LSB 1 schreiben
 
	sbi		RF_PORT, SCK		;Clock-Impuls erzeugen
	nop					
	nop
	cbi		RF_PORT, SCK

	dec		temp				;Bit Zhler runterzhlen
	brne	rf12_trans_loop		;Wenn nicht abgelaufen, wiederholen

	sbi		RF_PORT, CS			;CS auf 1(inaktiv)
	cbi		RF_PORT, SDI		;SDI auf 0
ret

;------ rf12_init ------------------------------------------------------------
;
;	Initialisierung des RFM12
;
;-------------------------------------------------------------------------------
RF12_init:
	ldi		temp, (1<<SDI)|(1<<SCK)|(1<<CS)
	out		RF_DDR, temp
	ldi		temp, (1<<CS)
	out		RF_PORT, temp

	rcall	wait_500ms						;wait until POR done
	
	ldiw	X, 0xC0E0						; CR16, AVR CLK: 10MHz
	rcall	rf12_trans						; CR = Control Register
	ldiw	X, 0x80E7						; CR1, el on, ef (FIFO) on, b1+b0=10 fr 868
	rcall	rf12_trans
	ldiw	X, 0xC2AB						; CR6, Data Filter: internal
	rcall	rf12_trans
	ldiw	X, 0xCA81						; CR7, Set FIFO mode
	rcall	rf12_trans
	ldiw	X, 0xE000						; CR14, disable wakeuptimer
	rcall	rf12_trans
	ldiw	X, 0xC800						; CR15, disable low duty cycle
	rcall	rf12_trans
	ldiw	X, 0xC4F7						; CR10, AFC settings: autotuning: -10kHz...+7,5kHz
	rcall	rf12_trans
	ldiw	X, 0xA6F8						; CR3, Frequency Setting Command
	rcall	rf12_trans						; Sende/OKcntsfrequenz auf 868,92MHz einstellen
	ldiw	X, 0x948C						; CR5, Receiver Control Command
	rcall	rf12_trans						; 200 kHz Bandbreite, -6dB Verstrkung, DRSSI threshold -79dBm
	ldiw	X, 0xC610						; CR4, Data Rate Command
	rcall	rf12_trans						; Baudrate 19200
	ldiw	X, 0x9870						; CR11, TX Configuration Control Command
	rcall	rf12_trans						; Ausgangangsleistung 6mW, Frequenzshift 120kHz
ret

;----------rf12_ready----------------------------------------------------------
rf12_ready:
	cbi		RF_PORT, SDI
	cbi		RF_PORT, CS			;CS auf 0(aktiv)
	nop					

rf12_wait:
	sbis	RF_PIN, SDO			;Wenn SDO noch nicht 1 ist
	rjmp	rf12_wait			;nochmal schaun, ob es 1 ist
ret

;------ rf12_rxdata ------------------------------------------------------------
;
;	Holt Inhalt vom OKcnts-FIFO nach XL
;
;-------------------------------------------------------------------------------
rf12_rxdata:
	ldiw	X, 0x82C8			;CR2, Power management Command, RX on
	rcall	rf12_trans			;er, ebb on, et, es off, ex on
	ldiw	X, 0xCA81			;CR7, FIFO and Reset Mode
	rcall	rf12_trans			;NON-sensitive reset
	ldiw	X, 0xCA83			;CR7, FIFO and Reset Mode, enable FIFO
	rcall	rf12_trans			;FIFO fill enabled after synchron
ret

rf12_rxdata_rec:
	rcall	rf12_ready
	ldiw	X, 0xB000		
	rcall	rf12_trans
	st		Y+, XL				;eingegangenes Byte speichern
	inc		cnt
	cp		cnt, datacnt
	breq	rf12_rxdata_exit
	rjmp	rf12_rxdata_rec		
rf12_rxdata_exit:
	clr		cnt
ret

rf12_rxdata_end:
	ldiw	X, 0xCA81			;CR7, FIFO and Reset Mode, enable FIFO
	rcall	rf12_trans			;FIFO fill enabled after synchron
	ldiw	X, 0x8208			;RX off
	rcall	rf12_trans
ret
