/*
 * RFM12_TX.asm
 * Sender fr RFM12
 *
 *  Created: 19.07.2015 18:17:05
 *  Author: BrunoW7
 *	in einer Schleife wird auf Empfang gewartet. Werden Daten empfangen,  
 *  wird auf bereinstimmung mit einem Muster geprft. Wenn 2 Datenstze
 *  richtig empfangen werden, wird der Befehl ausgefhrt. 
*/
;       			 ATMega48 
;   				+--+-+--+                        
;	    (/RESET)PC6	|  |_|	|	PC5(ADC5 SCL)                       
;   	   (RXD)PD0	|		|	PC4(ADC4 SDA)                    
;   	   (TXD)PD1	|		|	PC3(ADC3)               
;	      (INT0)PD2 |       |   PC2(ADC2) 	
;	      (INT1)PD3 |       |   PC1(ADC1) 	
;       (XCK T0)PD4 |       |   PC0(ADC0) 	
;	    		VCC |       |   GND
;   			GND |       |   AREF
;  (XTAL1 TOSC1)PB6 |		|	AVcc            
;  (XTAL2 TOSC2)PB7 |		|	PB5(SCK)            
;		    (T1)PD5	|		|	PB4(MISO)               
;		  (AIN0)PD6	|		|	PB3(MOSI OC2)                      
;		  (AIN1)PD7	|		|	PB2(/SS OC1B)                      
;	      (ICP1)PB0 |		|	PB1(OC1A)                  
;   				+-------+                           
; 
; PB0	I	nc (pullup on)
; PB1	I	nc (pullup on)
; PB2	I	RFM12-SDO 
; PB3	I	MOSI, RFM12_SDI
; PB4	I	MISO, RFM12-SCK
; PB5	I	SCK, RFM12-CS
; PB6	I	XTAL1, RFM12-CLK
; PB7	I	XTAL2  
; PC0 	I	nc (pullup on) 
; PC1 	I	nc (pullup on) 
; PC2	I	nc (pullup on) 
; PC3	I	nc (pullup on) 
; PC4	I	nc (pullup on) 
; PC5	I	nc (pullup on) 
; PC6	I	RESET, RFM12-NRES
; PD0	I	nc (pullup on) 
; PD1	I	nc (pullup on) 
; PD2	I	RFM12-NIRQ 
; PD3	I	nc (pullup on) 
; PD4	I	nc (pullup on)    
; PD5	I	nc (pullup on) 
; PD6	I	LED 
; PD7	I	nc (pullup on) 

.include "m48def.inc"

.DSEG
	Eingang: .BYTE 12

.CSEG
.equ F_CPU		= 8000000

.equ BAUD		= 56000	          ; Baudrate des UART
 
;Portbelegung
.EQU RF_PORT	= PORTB
.EQU RF_DDR		= DDRB
.EQU RF_PIN		= PINB

.EQU AUX_PORT	= PORTD
.EQU AUX_DDR	= DDRD

.EQU SDO		= 2
.EQU SDI		= 3
.EQU SCK		= 4
.EQU CS			= 5
.EQU LED		= 6

.def temp		= r16
.def temp1		= r17
.def temp2		= r18
.def cnt		= r19
.def datacnt	= r20
.def status		= r21
.def OKcnt_1	= r22
.def OKcnt_2	= r23

.macro  ldiw								;16-Bit Adresse aus Ram laden
  ldi  @0L,low(@1)
  ldi  @0H,high(@1)
.endmacro

.cseg 
 
;Init
start:
    ldi     temp, LOW(RAMEND)
    out     SPL, temp
    ldi     temp, HIGH(RAMEND)
    out     SPH, temp
 	
	ldi		temp, (1<<LED)
	out		AUX_DDR, temp


	rcall	UART_init
	rcall	RF12_init
	rcall	rf12_rxdata
	clr		OKcnt_1
	clr		OKcnt_2

main:
	clr		cnt
	cpi		OKcnt_1, 2						;bereits 2 Befehle eingegangen?
	breq	Sprung_1						;wenn ja, zur Aktion 1 springen
	cpi		OKcnt_2, 2						;bereits 2 Befehle eingegangen?
	breq	Sprung_2						;wenn ja, zur Aktion 2 springen
	sbic	RF_PIN, SDO
	inc		status
	tst		status
	breq	main
	clr		status
;Empfnger abfragen
receive:
	ldiw	Y, Eingang					;Y -Pointer auf 1. Byte
	ldi		datacnt, 10					;Anzahl Datenbytes +4
	rcall	rf12_rxdata_rec				;10 empfangene Bytes werden im SRAM abgelegt
;prfen, ob Startbyte = S
S_test:
	ldiw	Y, Eingang					;Y - Pointer wieder auf Anfang setzen
S_test_loop:	
	ld		temp, Y+					;Y - Byte laden und auf nchstes Byte stellen
	rcall	uart_transmit
	cpi		temp, 'S'
	breq	Vergleich					;wenn S, dann die brigen 6 Byte vergleichen
	inc		cnt
	cpi		cnt, 3						;S mu in den ersten 3 Bytes enthalten sein
	breq	main
rjmp		S_test_loop

;weitere 6 Bytes berprfen
Vergleich:
	clr		cnt
	ldi		ZL, LOW(Befehl_2*2)			;Z - Pointer auf Befehlsmuster
	ldi		ZH, HIGH(Befehl_2*2)
Ver_loop:
	inc		cnt
	cpi		cnt, 7						;6 Bytes mssen verglichen werden
	breq	Ver_loop_exit
	lpm									;Z - Byte laden und Register erhhen
	adiw	ZL, 1							
	ld		temp, Y+					;Y - Byte laden und auf nchstes Byte stellen
	;rcall	wait
	;rcall	wait
	rcall	uart_transmit
	cp		r0, temp					;Y mit Z vergleichen
	breq	Ver_loop					;wenn gleich, nchstes Byte
	rjmp	Fehler
	;rjmp	Vergleich_2						
Ver_loop_exit:
	clr		cnt
	inc		OKcnt_1						;Zhler fr erfolgreichen Vergleich
	rjmp	Clear						;Daten im SRAM lschen
rjmp		main

Sprung_1:
	rjmp	Aktion_1
Sprung_2:
	rjmp	Aktion_2

Clear:
	ldiw	Y, Eingang
	clr		temp
	st		Y+, temp
	inc		cnt
	cpi		cnt, 10
	breq	main
rjmp		Clear

Aktion_1:
	rcall	rf12_rxdata_end
	clr		OKcnt_1
	rcall	UART_Aktion_1						;ja, dann Aktion ausfhren
	rcall	rf12_rxdata
rjmp		main

Aktion_2:
	rcall	rf12_rxdata_end
	clr		OKcnt_2
	rcall	UART_Aktion_2						;ja, dann Aktion ausfhren
	rcall	rf12_rxdata
rjmp		main

Fehler:
	rcall	UART_Fehler
	clr		OKcnt_1
	clr		OKcnt_2
	clr		cnt
rjmp		main
	
_Fehler: 
	.db "Vergleichs-Fehler!!",10,13,0

_Aktion_1:
	.db "Auftrag 1 ausfuehren!!!", 10, 13,0

_Aktion_2:
	.db "Auftrag 2 ausfuehren!!!", 10, 13,0

Befehl_1:
	.db "XXAXXA"
Befehl_2:		
 	.db	"YYBYYB"

.include "RFM12-RX-868.inc"
.include "UART.inc"
.include "Hilfs.inc"
